@@ -156,14 +156,14 @@ static void cache_send_udp_command(struct uwsgi_cache *, char *, uint16_t, char
 static void cache_sync_hook(char *k, uint16_t kl, char *v, uint16_t vl, void *data) {
 	struct uwsgi_cache *uc = (struct uwsgi_cache *) data;
 	if (!uwsgi_strncmp(k, kl, "items", 5)) {
-		size_t num = uwsgi_str_num(v, vl);		
+		size_t num = uwsgi_str_num(v, vl);
 		if (num != uc->max_items) {
 			uwsgi_log("[cache-sync] invalid cache size, expected %llu received %llu\n", (unsigned long long) uc->max_items, (unsigned long long) num);
 			exit(1);
 		}
 	}
 	if (!uwsgi_strncmp(k, kl, "blocksize", 9)) {
-		size_t num = uwsgi_str_num(v, vl);		
+		size_t num = uwsgi_str_num(v, vl);
 		if (num != uc->blocksize) {
 			uwsgi_log("[cache-sync] invalid cache block size, expected %llu received %llu\n", (unsigned long long) uc->blocksize, (unsigned long long) num);
 			exit(1);
@@ -200,6 +200,40 @@ static void uwsgi_cache_load_files(struct uwsgi_cache *uc) {
 next:
 		usl = usl->next;
 	}
+
+#ifdef UWSGI_ZLIB
+	usl = uwsgi.load_file_in_cache_gzip;
+        while(usl) {
+                size_t len = 0;
+                char *value = NULL;
+                char *key = usl->value;
+                uint16_t key_len = usl->len;
+                char *space = strchr(usl->value, ' ');
+                if (space) {
+                        // need to skip ?
+                        if (uwsgi_strncmp(uc->name, uc->name_len, usl->value, space-usl->value)) {
+                                goto next2;
+                        }
+                        key = space+1;
+                        key_len = usl->len - ((space-usl->value)+1);
+                }
+                value = uwsgi_open_and_read(key, &len, 0, NULL);
+                if (value) {
+			struct uwsgi_buffer *gzipped = uwsgi_gzip(value, len);
+			if (gzipped) {
+                        	uwsgi_wlock(uc->lock);
+                        	if (!uwsgi_cache_set2(uc, key, key_len, gzipped->buf, gzipped->len, 0, 0)) {
+                                	uwsgi_log("[cache-gzip] stored \"%.*s\" in \"%s\"\n", key_len, key, uc->name);
+                        	}
+                        	uwsgi_rwunlock(uc->lock);
+				uwsgi_buffer_destroy(gzipped);
+			}
+                        free(value);
+                }
+next2:
+                usl = usl->next;
+        }
+#endif
 }
 
 
@@ -382,6 +416,28 @@ char *uwsgi_cache_get2(struct uwsgi_cache *uc, char *key, uint16_t keylen, uint6
 	return NULL;
 }
 
+char *uwsgi_cache_get3(struct uwsgi_cache *uc, char *key, uint16_t keylen, uint64_t * valsize, uint64_t *expires) {
+
+        uint64_t index = uwsgi_cache_get_index(uc, key, keylen);
+
+        if (index) {
+                struct uwsgi_cache_item *uci = cache_item(index);
+                if (uci->flags & UWSGI_CACHE_FLAG_UNGETTABLE)
+                        return NULL;
+                *valsize = uci->valsize;
+		if (expires)
+			*expires = uci->expires;
+                uci->hits++;
+                uc->hits++;
+                return uc->data + (uci->first_block * uc->blocksize);
+        }
+
+        uc->miss++;
+
+        return NULL;
+}
+
+
 int uwsgi_cache_del2(struct uwsgi_cache *uc, char *key, uint16_t keylen, uint64_t index, uint16_t flags) {
 
 	struct uwsgi_cache_item *uci;
@@ -427,6 +483,10 @@ int uwsgi_cache_del2(struct uwsgi_cache *uc, char *key, uint16_t keylen, uint64_
 		uci->expires = 0;
 
 		uc->n_items--;
+
+		if (uc->use_last_modified) {
+			uc->last_modified_at = uwsgi_now();
+		}
 	}
 
 	if (uc->nodes && ret == 0 && !(flags & UWSGI_CACHE_FLAG_LOCAL)) {
@@ -473,6 +533,7 @@ int uwsgi_cache_set2(struct uwsgi_cache *uc, char *key, uint16_t keylen, char *v
 	uint8_t rollback_mode = 0;
 
 	int ret = -1;
+	time_t now = 0;
 
 	if (!keylen || !vallen)
 		return -1;
@@ -518,13 +579,13 @@ int uwsgi_cache_set2(struct uwsgi_cache *uc, char *key, uint16_t keylen, char *v
 					uc->unused_blocks_stack_ptr++;
 				}
 				else if (rollback_mode == 2) {
-					uc->first_available_block--;	
+					uc->first_available_block--;
 				}
                                 goto end;
 			}
 			// mark used blocks;
-			uint64_t needed_blocks = cache_mark_blocks(uc, uci->first_block, vallen);	
-			// optimize teh scan
+			uint64_t needed_blocks = cache_mark_blocks(uc, uci->first_block, vallen);
+			// optimize the scan
 			if (uc->blocks_bitmap_pos + (needed_blocks+1) > uc->blocks) {
                         	uc->blocks_bitmap_pos = 0;
                         }
@@ -532,8 +593,10 @@ int uwsgi_cache_set2(struct uwsgi_cache *uc, char *key, uint16_t keylen, char *v
                         	uc->blocks_bitmap_pos = uci->first_block + needed_blocks + 1;
                         }
 		}
-		if (expires && !(flags & UWSGI_CACHE_FLAG_ABSEXPIRE))
-			expires += uwsgi_now();
+		if (expires && !(flags & UWSGI_CACHE_FLAG_ABSEXPIRE)) {
+			now = uwsgi_now();
+			expires += now;
+		}
 		uci->expires = expires;
 		uci->hash = uc->hash->func(key, keylen);
 		uci->hits = 0;
@@ -572,7 +635,8 @@ int uwsgi_cache_set2(struct uwsgi_cache *uc, char *key, uint16_t keylen, char *v
 	else if (flags & UWSGI_CACHE_FLAG_UPDATE) {
 		uci = cache_item(index);
 		if (expires && !(flags & UWSGI_CACHE_FLAG_ABSEXPIRE)) {
-			expires += uwsgi_now();
+			now = uwsgi_now();
+			expires += now;
 			uci->expires = expires;
 		}
 		if (uc->blocks_bitmap) {
@@ -601,7 +665,11 @@ int uwsgi_cache_set2(struct uwsgi_cache *uc, char *key, uint16_t keylen, char *v
 		uci->valsize = vallen;
 		ret = 0;
 	}
-	
+
+	if (uc->use_last_modified) {
+		uc->last_modified_at = (now ? now : uwsgi_now());
+	}
+
 	if (uc->nodes && ret == 0 && !(flags & UWSGI_CACHE_FLAG_LOCAL)) {
 		cache_send_udp_command(uc, key, keylen, val, vallen, expires, 10);
 	}
@@ -879,6 +947,7 @@ struct uwsgi_cache *uwsgi_cache_create(char *arg) {
 		uc->nodes = uwsgi.cache_udp_node;
 		uc->udp_servers = uwsgi.cache_udp_server;
 		uc->store_sync = uwsgi.cache_store_sync;
+		uc->use_last_modified = (uint8_t) uwsgi.cache_use_last_modified;
 
 		if (uwsgi.cache_sync) {
 			uwsgi_string_new_list(&uc->sync_nodes, uwsgi.cache_sync);
@@ -898,6 +967,7 @@ struct uwsgi_cache *uwsgi_cache_create(char *arg) {
 		char *c_sync = NULL;
 		char *c_udp_servers = NULL;
 		char *c_bitmap = NULL;
+		char *c_use_last_modified = NULL;
 
 		if (uwsgi_kvlist_parse(arg, strlen(arg), ',', '=',
                         "name", &c_name,
@@ -923,6 +993,7 @@ struct uwsgi_cache *uwsgi_cache_create(char *arg) {
                         "udpservers", &c_udp_servers,
                         "udpserver", &c_udp_servers,
                         "bitmap", &c_bitmap,
+                        "lastmod", &c_use_last_modified,
                 	NULL)) {
 			uwsgi_log("unable to parse cache definition\n");
 			exit(1);
@@ -967,6 +1038,7 @@ struct uwsgi_cache *uwsgi_cache_create(char *arg) {
 			uc->use_blocks_bitmap = 1; 
 			uc->max_item_size = uc->blocksize * uc->blocks;
 		}
+		if (c_use_last_modified) uc->use_last_modified = 1;
 
 		uc->store_sync = uwsgi.cache_store_sync;
 		if (c_store_sync) { uc->store_sync = uwsgi_n64(c_store_sync); }
@@ -1233,7 +1305,7 @@ static int cache_magic_send_and_manage(int fd, struct uwsgi_buffer *ub, char *st
 	return 0;
 }
 
-char *uwsgi_cache_magic_get(char *key, uint16_t keylen, uint64_t *vallen, char *cache) {
+char *uwsgi_cache_magic_get(char *key, uint16_t keylen, uint64_t *vallen, uint64_t *expires, char *cache) {
 	struct uwsgi_cache_magic_context ucmc;
 	struct uwsgi_cache *uc = NULL;
 	char *cache_server = NULL;
@@ -1258,7 +1330,7 @@ char *uwsgi_cache_magic_get(char *key, uint16_t keylen, uint64_t *vallen, char *
 	// we have a local cache !!!
 	if (uc) {
 		uwsgi_rlock(uc->lock);
-		char *value = uwsgi_cache_get2(uc, key, keylen, vallen);
+		char *value = uwsgi_cache_get3(uc, key, keylen, vallen, expires);
 		if (!value) {
 			uwsgi_rwunlock(uc->lock);
 			return NULL;
@@ -1328,6 +1400,9 @@ char *uwsgi_cache_magic_get(char *key, uint16_t keylen, uint64_t *vallen, char *
 		ub->buf = NULL;
 		uwsgi_buffer_destroy(ub);
 		*vallen = ucmc.size;
+		if (expires) {
+			*expires = ucmc.expires;
+		}
 		return value;
 		
 	}
@@ -1625,7 +1700,6 @@ int uwsgi_cache_magic_clear(char *cache) {
 
 void uwsgi_cache_sync_from_nodes(struct uwsgi_cache *uc) {
 	struct uwsgi_string_list *usl = uc->sync_nodes;
-	usl = uc->sync_nodes;
 	while(usl) {
 		uwsgi_log("[cache-sync] getting cache dump from %s ...\n", usl->value);
 		int fd = uwsgi_connect(usl->value, 0, 0);
