@@ -208,18 +208,20 @@ void uwsgi_spawn_daemon(struct uwsgi_daemon *ud) {
 			uwsgi_write_pidfile(ud->pidfile);
 		}
 
-		// /dev/null will became stdin
-		devnull = open("/dev/null", O_RDONLY);
-		if (devnull < 0) {
-			uwsgi_error("/dev/null open()");
-			exit(1);
-		}
-		if (devnull != 0) {
-			if (dup2(devnull, 0) < 0) {
-				uwsgi_error("dup2()");
+		if (!uwsgi.daemons_honour_stdin) {
+			// /dev/null will became stdin
+			devnull = open("/dev/null", O_RDONLY);
+			if (devnull < 0) {
+				uwsgi_error("/dev/null open()");
 				exit(1);
 			}
-			close(devnull);
+			if (devnull != 0) {
+				if (dup2(devnull, 0) < 0) {
+					uwsgi_error("dup2()");
+					exit(1);
+				}
+				close(devnull);
+			}
 		}
 
 		if (setsid() < 0) {
