@@ -485,6 +485,28 @@ static int uwsgi_router_goon(struct uwsgi_route *ur, char *arg) {
 	return 0;
 }
 
+// flush response
+static int transform_flush(struct wsgi_request *wsgi_req, struct uwsgi_buffer *ub, struct uwsgi_buffer **foobar, void *data) {
+	wsgi_req->flush = 1;
+	return uwsgi_response_write_body_do(wsgi_req, ub->buf, ub->pos);
+        return 0;
+}
+static int uwsgi_router_flush_func(struct wsgi_request *wsgi_req, struct uwsgi_route *route) {
+	// if there are no transformations defined, set the flush right now
+	if (!wsgi_req->transformations) {
+		wsgi_req->flush = 1;
+	}
+	else {
+		// otherwise push it in the chain
+		uwsgi_add_transformation(wsgi_req, transform_flush, NULL);
+	}
+	return UWSGI_ROUTE_NEXT;	
+}
+static int uwsgi_router_flush(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_flush_func;
+        return 0;
+}
+
 // log route
 static int uwsgi_router_log_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
 
@@ -1167,9 +1189,14 @@ static char *uwsgi_route_var_uwsgi(struct wsgi_request *wsgi_req, char *key, uin
 }
 
 static char *uwsgi_route_var_mime(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
-	size_t mime_type_len = 0;
-        char *ret = uwsgi_get_mime_type(key, keylen, &mime_type_len);
-	if (ret) *vallen = mime_type_len;
+	char *ret = NULL;
+        uint16_t var_vallen = 0;
+        char *var_value = uwsgi_get_var(wsgi_req, key, keylen, &var_vallen);
+        if (var_value) {
+		size_t mime_type_len = 0;
+		char *ret = uwsgi_get_mime_type(key, keylen, &mime_type_len);
+		if (ret) *vallen = mime_type_len;
+        }
         return ret;
 }
 
@@ -1271,6 +1298,8 @@ void uwsgi_register_embedded_routers() {
         uwsgi_register_router("setprocname", uwsgi_router_setprocname);
         uwsgi_register_router("alarm", uwsgi_router_alarm);
 
+        uwsgi_register_router("flush", uwsgi_router_flush);
+
         uwsgi_register_route_condition("exists", uwsgi_route_condition_exists);
         uwsgi_register_route_condition("isfile", uwsgi_route_condition_isfile);
         uwsgi_register_route_condition("isdir", uwsgi_route_condition_isdir);
