@@ -310,13 +310,14 @@ struct uwsgi_buffer *uwsgi_to_http(struct wsgi_request *wsgi_req, char *host, ui
         	if (uwsgi_buffer_append(ub, wsgi_req->uri, wsgi_req->uri_len)) goto clear;
 	}
 
+	// force HTTP/1.0
         if (uwsgi_buffer_append(ub, " HTTP/1.0\r\n", 11)) goto clear;
 
         int i;
 	char *x_forwarded_for = NULL;
 	size_t x_forwarded_for_len = 0;
 
-        // starting adding headers
+        // start adding headers
         for(i=0;i<wsgi_req->var_cnt;i++) {
 		if (!uwsgi_starts_with(wsgi_req->hvec[i].iov_base, wsgi_req->hvec[i].iov_len, "HTTP_", 5)) {
 
@@ -325,8 +326,17 @@ struct uwsgi_buffer *uwsgi_to_http(struct wsgi_request *wsgi_req, char *host, ui
 
 			if (host && !uwsgi_strncmp(header, header_len, "HOST", 4)) goto next;
 
+			// remove dangerous headers
 			if (!uwsgi_strncmp(header, header_len, "CONNECTION", 10)) goto next;
 			if (!uwsgi_strncmp(header, header_len, "KEEP_ALIVE", 10)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "ACCEPT_ENCODING", 15)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "TE", 2)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "TRAILER", 7)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "IF_MATCH", 8)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "IF_MODIFIED_SINCE", 17)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "IF_RANGE", 8)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "IF_UNMODIFIED_SINCE", 19)) goto next;
+			if (!uwsgi_strncmp(header, header_len, "IF_NONE_MATCH", 13)) goto next;
 			if (!uwsgi_strncmp(header, header_len, "X_FORWARDED_FOR", 15)) {
 				x_forwarded_for = wsgi_req->hvec[i+1].iov_base;
 				x_forwarded_for_len = wsgi_req->hvec[i+1].iov_len;
@@ -386,3 +396,44 @@ clear:
         return NULL;
 }
 
+int uwsgi_is_full_http(struct uwsgi_buffer *ub) {
+	size_t i;
+	int status = 0;
+	for(i=0;i<ub->pos;i++) {
+		switch(status) {
+			// \r
+			case 0:
+				if (ub->buf[i] == '\r') status = 1;
+				break;
+			// \r\n
+			case 1:
+				if (ub->buf[i] == '\n') {
+					status = 2;
+					break;
+				}
+				status = 0;
+				break;
+			// \r\n\r
+			case 2:
+				if (ub->buf[i] == '\r') {
+					status = 3;
+					break;
+				}
+				status = 0;
+				break;
+			// \r\n\r\n
+			case 3:
+				if (ub->buf[i] == '\n') {
+					return 1;
+				}
+				status = 0;
+				break;
+			default:
+				status = 0;
+				break;
+			
+		}
+	}
+
+	return 0;
+}
